/* 
Unlimited Simulator is copyrighted (c) Bruno Barberi Gnecco, 1999, and
distributed under the GNU General Public License (GPL). The conditions can be
found in the file GPL, resident at the main directory of the distribution.
*/
#include "server.h"
#include <stdio.h>
#include <string.h>

#define FILEPREF	"server.pref"

PREF_UL *prefs;

#define YES( c )	( c == 'y' || c == 'Y' )

/* for fields regarding other applications, like sound or video output, 0
if a numeric filed or '\0' if a string will set defaults */
/*** should detect errors with scanfs and etc */
int loadprefs ( void ) {
	FILE *input;
	char temp[512], *value;
	int i = 0;
	
	if ( !(prefs = (PREF_UL *)malloc(sizeof(PREF_UL))) ) {
		fprintf( stderr, "Couldn't allocate prefs! ARGH!\n" );
		return FATALERROR;
	}
	
	prefs->anytime = True;
	prefs->ai = True;
	prefs->clientclient = True;
	
	prefs->kickout = False;
	prefs->kickmajority = 66.6;
	
	prefs->maxmessages = 0;
	prefs->maxmesbytes = 0;
	prefs->maxobjcreate = 3;
	prefs->maxobjmodify = 3;
	prefs->maxobjmove = 5;
	
	prefs->maxsimtime.h = prefs->maxsimtime.min = 
		prefs->maxsimtime.sec = prefs->maxsimtime.usec = 0;

	sprintf( temp, "%s%s", ULDIR, FILEPREF );
	if ( (input = fopen( temp, "r" )) ) {
		while ( fgets( temp, sizeof( temp ), input )) {
			i++;
			
			if ( value = strchr( temp, '#' ) )
				*value = '\0';
			
			value = strchr( temp, '=' );
			if ( !value )
				continue;
			*value++ = '\0';

			while( isspace( *value ) )
				value++;

			if ( strstr( temp, "anytime" ) )
				prefs->anytime = ( YES(*value) ? True : False );
			else if ( strstr( temp, "ai" ) )
				prefs->ai = ( YES(*value) ? True : False );
			else if ( strstr( temp, "clientclient" ) )
				prefs->clientclient = ( YES(*value) ? True : False );
			else if ( strstr( temp, "kickout" ) )
				prefs->kickout = ( YES(*value) ? True : False );
			else if ( strstr( temp, "kickmajority" ) )
				prefs->kickmajority = atoi( value );
			else if ( strstr( temp, "maxmessages" ) )
				prefs->maxmessages = atoi( value );
			else if ( strstr( temp, "maxmesbytes" ) )
				prefs->maxmesbytes = atoi( value );
			else if ( strstr( temp, "maxobjcreate" ) )
				prefs->maxobjcreate = atoi( value );
			else if ( strstr( temp, "maxobjmodify" ) )
				prefs->maxobjmodify = atoi( value );
			else if ( strstr( temp, "maxobjmove" ) )
				prefs->maxobjmove = atoi( value );
			else if ( strstr( temp, "maxsimtime" ) )
				sscanf( value, "%d:%d:%d:%d", &prefs->maxsimtime.h,
					&prefs->maxsimtime.min, &prefs->maxsimtime.sec,
					&prefs->maxsimtime.usec );
			else
				fprintf( stderr, "Syntax error in %s, line %d; ignoring\n",
					 FILEPREF, i );
		}
		fclose( input );
		return OK;
	}
	else {
		fprintf( stderr, 
			"Could not open preferences file: %s. Trying to create a new one\n",
			FILEPREF );
		saveprefs();
		return OK;
	}
		
}

#define YESNO( d )		( d == True ? "yes" : "no" )

void saveprefs ( void ) {
	unsigned char temp[512];
	FILE *out;	

	sprintf( temp, "%s%s", ULDIR, FILEPREF );
	if ( (out = fopen( temp, "w" )) ) {
		fprintf( out, "# Unlimited Simulator server preferences file\n" );
		fprintf( out, "anytime = %s\n", YESNO( prefs->anytime ) );
		fprintf( out, "ai = %s\n", YESNO( prefs->ai ) );
		fprintf( out, "clientclient = %s\n", YESNO( prefs->clientclient ) );
		fprintf( out, "kickout = %s\n", YESNO( prefs->kickout ) );
		fprintf( out, "kickmajority = %f\n", prefs->kickmajority );
		fprintf( out, "maxmessages = %d\n", prefs->maxmessages );
		fprintf( out, "maxmesbytes = %d\n", prefs->maxmesbytes );
		fprintf( out, "maxobjcreate = %d\n", prefs->maxobjcreate );
		fprintf( out, "maxobjmodify = %d\n", prefs->maxobjmodify );
		fprintf( out, "maxobjmove = %d\n", prefs->maxobjmove );
		fprintf( out, "maxsimtime = %d:%d:%d:%d\n", prefs->maxsimtime.h,
			prefs->maxsimtime.min, prefs->maxsimtime.sec,
			prefs->maxsimtime.usec );
		fprintf( out, "# End of configuration file" );
		fclose( out );
	}
	else
		fprintf( stderr, "Could not open preferences file to save\n" );
}
