/* 
Unlimited Simulator is copyrighted (c) Bruno Barberi Gnecco, 1999, and
distributed under the GNU General Public License (GPL). The conditions can be
found in the file GPL, resident at the main directory of the distribution.
*/

/* This file, netopen.c, is adapted from original code by Steve Kirkendall.
Permission given. */

/* This file contains functions for opening a connection to a given port on a
 * given machine.
 *
 *   int netopen(char *site_port, int defport);
 *	Open a socket connection to a specific port on a specific site.
 *	site_port is a host name or numbers-and-dots address, optionally
 *	followed by a port number.  defport is the port number to use if
 *	site_port contains no port number.
 */
		 
#include "net.h"
#include <unistd.h>
#include <string.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <ctype.h>
#include <stdlib.h>

/* Find the address of a site.  The site can be given as either a domain name,
 * or a "numbers and dots" name.  If successful, the address is stuffed into
 * address struct, and TRUE is returned; otherwise FALSE is returned.
 */
static Bool site2addr ( char *site, struct in_addr *address ) {
	struct hostent	*siteinfo;
	unsigned long	localorder;

	/* if the site name starts with a digit, then assume it is in the
	 * "numbers and dots" format.
	 */
	if ( isdigit(site[0]) ) {
		/* convert to binary address */
		localorder = inet_addr(site);
		if ( localorder == INADDR_NONE )
			goto Error;
		address->s_addr = htonl(localorder);
	}
	else {
		/* look up the name */
		siteinfo = gethostbyname(site);
		if ( !siteinfo )
			goto Error;

		/* use the primary address */
		memcpy( address, siteinfo->h_addr_list[0], siteinfo->h_length );
	}

	return True;

Error:
	return False;
}



/* Open a connection to a given site and port.  If site_port contains a port
 * number, then that port number is used; otherwise defport is used.
 */
int netopen ( char *site_port, int defport ) {
	struct in_addr	serverAddress;
	struct sockaddr_in destPort;
	char		*tmp;
	char		buf[150];
	int		fd;

	/* if site_port contains a port number, then separate it from the site
	 * name, and use the given port instead of the default port.
	 */
	strcpy( buf, site_port );
	tmp = strchr( buf, ':' );
	if ( tmp ) {
		*tmp++ = '\0';
		defport = atoi( tmp );
	}

	/* look up the address of the server */
	if ( site2addr(buf, &serverAddress) == False )
		return -1;

	/* create a socket, and a sockbuf_t to buffer it */
	fd = socket( AF_INET, SOCK_STREAM, IPPROTO_IP );
	if ( fd < 0 )
		return -1;

	/* connect the socket to the remote machine */
	destPort.sin_family = AF_INET;
	destPort.sin_port = htons((unsigned short)defport);
	destPort.sin_addr = serverAddress;
	if ( connect(fd, (struct sockaddr *)&destPort, sizeof(destPort)) ) {
		netclose( fd );
		return -1;
	}

	return fd;
}
