/* 
Unlimited Simulator is copyrighted (c) Bruno Barberi Gnecco, 1999, and
distributed under the GNU General Public License (GPL). The conditions can be
found in the file GPL, resident at the main directory of the distribution.
*/
#ifndef UNLIMITED_H
#define UNLIMITED_H

#include "ulengine.h"
#include "time.h"
#include "net.h"

#define DEFPORT		1830

/* some general messages */
#define noobjcreate	"Object creation is not allowed in this server"
#define noobjdestroy	"Object destruction is not allowed in this server"
#define noobjmodify	"Object modification is not allowed in this server"

struct general_ul { /* these are basically shared preferences. Server rules. */
	dFloat		gravity;	/* by now Newton is enough. :) */
	dFloat		temperature;
	Bool		useobjso;
	Bool		createobject;	/* can objects be created? */
	Bool		destroyobject;	/* can objects be destroyed? */
	Bool		modifyobject;	/* can objects be modified? */
	Bool		rigidobjects;	/* objects (not vehicles) do not deform. */
};
typedef struct general_ul GENERAL_UL;

enum weather_ul {
	SUN, CLOUDY, RAIN, SNOW
};
typedef enum weather_ul WEATHER_UL;

#define WEATHERS 	4

enum raceflag_ul {
	GREEN, 		/* everything is fine */
	WHITE, 		/* last lap */
	CHECKED, 	/* race start/end */
	YELLOW,		/* accident on track */
	RED,		/* go to boxes */
	BLACK		/* out of race */
};
typedef enum raceflag_ul RACEFLAG_UL;

struct score_ul {
	sInt 		position;	/* which position in score */
	uByte		*driver;	/* driver's name */
	uByte		*model;		/* client used */
	WEATHER_UL	weather;
	TIME_UL		laptime;
};
typedef struct score_ul SCORE_UL;

typedef sInt TERRAINTYPE_UL;

struct terrain_ul {
	TERRAINTYPE_UL 	id;		/* this is the terrain id */
	uByte 	*name;
	dFloat 		friction[WEATHERS];	/* friction in tires */
	dFloat 		maxspeed;	/* in % of maxspeed. Asphalt is 100 */
#ifdef CLIENT
	ggi_color 	color;
	uByte 	*sound;		/* soundfile */
/*	Image 		*texture; */
#endif
	struct terrain_ul *next;
};
typedef struct terrain_ul TERRAIN_UL;

#define speed(client)	(sqrt(client->momentum.x*client->momentum.x+\
			  client->momentum.y*client->momentum.y+\
			  client->momentum.z*client->momentum.z)/client->mass)

enum status_ul {
	CONNECTING, READY, COUNTDOWN, RACING, PAUSED, QUIT
};
typedef enum status_ul STATUS_UL;

#ifndef SERVER
/* spinning is when the client is spinning */
enum tirestate_ul {
	NORMAL, SPINNING, BURNING, OUTOFGROUND, PUNCTURED, BLOWN
};
typedef enum tirestate_ul TIRESTATE_UL;

/* wheels are defined here because they're widely used in many vehicles,
and faster to draw using ellipses than polygons. They will be moved to a
library someday. */
struct wheel_ul {
	TIRESTATE_UL	state;		/* how's wheel? */
	TERRAIN_UL 	*terrain;	/* which terrain is it in? */
	sInt		wearing;	/* 1 000 000 is new, 0 is destroyed */
	fFloat		radius, circ;	/* radius, circumference in meters */
	fFloat		rotation;	/* in rad/s */
	fFloat		width;		/* in meters */
	fFloat		angle;		/* steering of wheels, in radians. 
					   Straight = 0, to left minus */
	fFloat 		maxsteer;	/* maximum steering, in radians */
	VECTOR_UL	normal;		/* ground force at the wheel */
	fFloat		temperature;	/* kelvin */
};
typedef struct wheel_ul WHEEL_UL;

/* This enum will futurely hold every event that can happen in the game. */
enum event_ul {
	KEYPRESS_UL, KEYREPEAT_UL, KEYRELEASE_UL
};
typedef enum event_ul EVENT_UL;
#endif

struct simpleclient_ul {
	VECTOR_UL 	position, direction;	/* received */
	dFloat		angle;
	VECTOR_UL 	momentum, angmomentum;	/* calculated here */
	STATUS_UL	status;		/* racing? It may have quited, or
					  maybe not racing. */
	uLong		id;

	fFloat		mass;		/* received. once? */
	sInt 		racepos;	/* in race */

#ifndef NONE
	ggi_color	color;		/* of client */
#endif

	uByte		*driver;
	uByte		*model;		/* name of the client file, no extension */
	DOODAD_UL	*geom;		/* this is not downloaded, at least by now */
	Bool 		ai;		/* AI client? */

#ifdef SERVER /* by now, only server uses this */
	sInt		sock;
	
	/* network preferences */
	sInt		updatespersec;	/* number of updates of position per
					   second */
	TIMESTAMP_UL	secsperupdate;	/* inverse of last field */
	Bool		directconnect;	/* do client accept direct connections
					   to other clients? */
	dFloat		maxsqdist;	/* things farther than sqrt(maxdist) are not
					  updated so often */
	UPDATE_UL	updates;	
#endif
	
	struct simpleclient_ul *next;
};
typedef struct simpleclient_ul SIMPLECLIENT_UL;

/* these are objects in track */
struct object_ul {
	VECTOR_UL	position, direction;
	dFloat		angle;
#ifdef SERVER
	Bool 		isstatic;
#endif
	VECTOR_UL 	momentum, angmomentum;
	fFloat		mass;
	uByte		*name;
	uLong		id;		/* this is a unique identifier of
	   	objects and clients. Client's id is defined as: (number<<16)
		To avoid more than one client to create objects with the
		same ID, the first half of the id (id&0xFFFF0000) is the 
		client's number, and the second part is a counter defined
		statically in io.c, which is updated upon the creation
		of a new object. Note that the first object is 
		(number<<16)+1. */
	DOODAD_UL 	*geom;
	
	void		*struc;		/* object's own structure, if it
					   has one. */
	void		*lib;		/* external library */
	sInt		(*init)( struct object_ul * );
	void		(*finish)( struct object_ul * );
	sInt		(*brain)( struct object_ul * );

	struct object_ul *next;
};
typedef struct object_ul OBJECT_UL;

enum orient_ul {
	NORTH, EAST, SOUTH, WEST	
};
typedef enum orient_ul ORIENT_UL;

/* components of a trackway */
struct way_ul {
	VECTOR_UL	point;
	dFloat		width;		/* track width */
	TERRAIN_UL	*terrain;
	VECTOR_UL	tan;		/* tangent vectors */
	VECTOR_UL	normal;		/* normal vectors */
	Bool 		checkpoint; 
};
typedef struct way_ul WAY_UL;

/* the track structure */
struct track_ul {
	uByte 	*name, *author, *date, *comment;
	fFloat 		width, length;	/* of terrain, in meters */
	fFloat 		extension;	/* of a lap, in meters */

	SCORE_UL 	besttimes[5];	/* best 5 scores */
	VECTOR_UL 	startline[2];	/* start/end points */
	
	sInt 		totallap;	/* total number of laps */
	sInt 		lap;		/* number of laps completed by leader */
	
	TIME_UL		best;		/* best time of race */
	sInt 		bestclient;	/* number of client that made it */
	sInt 		temperature;	/* of weather, 10*K */
	/*** what about a distribution of temperatures? */

#ifdef CLIENT
/*	Image		*map;	*/	/* little map of the track to the user */
/*	Image 		*horizon;*/	/* some image to be placed as horizon.
				 should be mapped in a cilinder around track*/
	FOG_UL 		fog;
#endif
	
	Bool 		useflag;	/* use flags? */
	RACEFLAG_UL	rflag;		/* which flag is on */
	WEATHER_UL 	weather;	/* weather */
	
	DOODAD_UL	*geom;
	WAY_UL		*ways;		/* the real trackway, like roads, etc */
	
	dFloat		checkradius;	/* radius of each checkpoint.
					   I think this check should be made
					   in the server, to avoid cheating... */
};
typedef struct track_ul TRACK_UL;

#ifndef SERVER
struct client_ul {
	VECTOR_UL	position; 	/* client's position (m)*/
	VECTOR_UL	direction; 	/* client's position (m)*/
	dFloat		angle;
	/* we could make the two above a quaternion... */	
	VECTOR_UL	momentum;	/* client's momentum (kg.m/s) */
	VECTOR_UL	angmomentum;	/* angular momentum (kg.m^2/s) */
	VECTOR_UL	oldpos;		/* to send delta(pos) to other racers */
	VECTOR_UL	box;		/* paralelepiped around client. Note that
				   client's center of mass is 0,0,0 in local coord,
				   so we need to define just the distance to
				   the center of three faces */
	fFloat		mass;		/* (kg) */
	fFloat		weight;		/* (Newtons), mass*G */
	sInt 		speed;		/* instantaneous speed (|momentum|/mass)(m/s) */
	WHEEL_UL 	*wheels;	/* 0 is right front, count clockwise,
					   if any */
			
	/* miscellaneous text */				   
	uByte	*model;		/* model, i.e., what's this? */
	uByte	*driver;	/* name of the client's controller */
	uLong	id;
	STATUS_UL	status;

	ggi_color	color;

	/* if we're racing, then these fields are useful */
	TIME_UL 	lastlap;	/* last lap time */
	TIME_UL 	elapsed;	/* elapsed time since beginning of race */
	TIME_UL 	bestlaptime;	/* best lap time */
	sInt 		bestlap;	/* number of best lap */
	sInt		rank;		/* position in race */
	sInt 		laps;		/* completed laps */

	/* client structure */
	DOODAD_UL	*geom;		/* server will use for coldet */
	Bool 		isai;		/* is client AI controlled? */

	void		*struc;		/* pointer to your own structure,
					   containing all info about this
					   client that it's usually only 
					   relevant to itself */
	void 		*lib;		/* client's dynamic library */
	sInt		(*init)( void );
	void		(*finish)( void );
	sInt		(*event)( EVENT_UL, int, int );
	sInt		(*loop)( void );
	sInt 		(*message)( CLIENTM_UL *, SERVERM_UL * );

#ifdef CAR
	/* these are the old fields of a car. they should be placed in a
	  car's simulation code. */
	/* funcs */
	void 		(*AirFriction)( struct car_ul *, VECTOR_UL *, VECTOR_UL * );
	void 		(*WheelFriction)( WHEEL_UL *, VECTOR_UL *, VECTOR_UL * );
	void 		(*Engine)( struct car_ul *, USERINPUT_UL *, VECTOR_UL * );
	void 		(*Suspension)( void );
	void 		(*Gear)( struct car_ul *, USERINPUT_UL * );
	sByte		gear, maxgear;	/* 0 is park, -1 is reverse */
	fFloat 		tank;		/* size (liters) */
	fFloat 		fuel;		/* (m^3)=(10^3)*l*/
	fFloat 		rotation;	/* engine rotation (rad/s)=(60/2PI)rpm */
	fFloat		temperature;	/* engine temperature, (Kelvin) */
	/* car specs */
	sInt 		maxspeed;	/* m/s, undamaged, in plane & horizontal
					   asphalt */
	dFloat 		area;		/* for air friction */
	dFloat 		maxpower;	/* in watts */
	dFloat 		drag;		/* usually from 0.2 to 0.6, dimensionless */
#endif
};
typedef struct client_ul CLIENT_UL;
#endif

/* init.c */
extern GENERAL_UL general;
extern sInt 	initGeneral ( void );
extern void	saveGeneral ( void );

/* io.c */
extern TERRAIN_UL		*firstterrain;
extern OBJECT_UL		*firstobj;
extern sInt findTrack 		( uByte *trackname );
extern TRACK_UL *readTrack	( uByte *filename );
extern sInt saveScores		( TRACK_UL *track );
extern void closeTrack		( TRACK_UL *track );
extern sInt readTerrain 	( sInt id, uByte *name );
extern void closeTerrains	( void );
extern sInt intnofunc		( OBJECT_UL *obj );
extern void voidnofunc		( OBJECT_UL *obj );
extern sInt findClient		( uByte *clientname );
#ifndef SERVER
extern CLIENT_UL *readClient	( uByte *name );
extern OBJECT_UL *readObject	( uLong clientid, uByte *name,
					 VECTOR_UL *pos, VECTOR_UL *dir );
#else
extern OBJECT_UL *readObject	( uLong clientid, uLong id, 
					uByte *name, VECTOR_UL *pos, VECTOR_UL *dir );
#endif
extern OBJECT_UL *copyObject	( OBJECT_UL *obj, VECTOR_UL *dir, VECTOR_UL *pos,
					dFloat angle );
extern void	freeObject	( OBJECT_UL *obj );

/* lsq.c  */
extern sInt	lsq ( sInt n, sInt m, dFloat x[n], dFloat y[m], dFloat f[n][m],
		     dFloat startx, dFloat starty, dFloat endx, dFloat endy, 
		     sInt u, sInt v, dFloat output[u][v] );

/* ps.c 
#ifndef SERVER
extern void ps ( CLIENT_UL *client, USERINPUT_UL *ui, dFloat frametime );
 old car stuff 
extern void defAirFriction ( CAR_UL *car, VECTOR_UL *force, VECTOR_UL *torque );
extern void defWheelFriction ( WHEEL_UL *wheel, VECTOR_UL *force, VECTOR_UL *torque );
extern void defEngine ( CAR_UL *car, USERINPUT_UL *ui, VECTOR_UL *force );
extern void defSuspension ( void );
extern void defGear ( CAR_UL *car, USERINPUT_UL *ui ); 
#endif */

#endif
