/* 
Unlimited Simulator is copyrighted (c) Bruno Barberi Gnecco, 1999, and
distributed under the GNU General Public License (GPL). The conditions can be
found in the file GPL, resident at the main directory of the distribution.
*/

#ifdef CLIENT
#include "ulengine.h" 

/* clips poly against camera's frustum. Returns 0 if not clipped */
int clipPolyC ( CAMERA_UL *camera, POLY_UL *poly, POLY_UL *result ) {
	int i, r = 0;
	PLANE_UL *planes[7];

	for ( i = 0; i < 4; i++ )
		planes[i] = &camera->frustrum.side[i];
	
	if ( camera->znear ) 
		planes[i++] = &camera->frustrum.znear;

	if ( camera->zfar ) 
		planes[i++] = &camera->frustrum.zfar;
	planes[i] = NULL;
	
	clipPolyP( planes, poly, result );

	return r;
}

/* clip should be done in 3D, this way: get vertex x(i) and x(i+1), check if
x(i)-x(i+1) intersect any of the planes (it can intersect 0, 1 or 2 planes
as it is a segment and not a line), find the points and substitute them by
(x(i), x(i+1)) respectively */
/* comments: this function: 
* will blow if two points are parallel to the plane */

/************ correct: check previous/next stuff, specialy in the result
calculations, chekc the correct order of point[j]-point[j+1] or the inverse */
int clipPolyP ( PLANE_UL **plane, POLY_UL *poly, POLY_UL *result ) {
	int i, j, k = 0, r = 0;
	PLANE_UL *temp;
	VECTOR_UL dir;
	double l, h;
	
	for ( j = 0; j < poly->npoints; j++ ) {
		for ( i = 0; plane[i] != NULL; i++ ) {
			h = plane[i]->a*poly->points[j].vect->x +
			    plane[i]->b*poly->points[j].vect->y +
			    plane[i]->c*poly->points[j].vect->z;
			if ( h + plane[i]->d < 0 ) {
				/* point is on wrong side of the plane, so 
				 it'll be replaced by the intersection */

#define PREV(j)		( j == 0 ? poly->npoints-1 : j+1 )
#define NEXT(j)		( j == poly->npoints-1 ? 0 : j+1 )

				/* first, with the previous vertex */
				/* get direction of straight line */
				subV( &poly->points[PREV(j)].vect,
					&poly->points[j].vect, &dir );
				/* l comes from: X = (x0,y0,z0) + l(a,b,c) */
				l = (-plane[i]->d - h)/(plane[i]->a*dir.x +
					plane[i]->b*dir.y + plane[i]->c*dir.z);
				result->points[k].vect->x = dir.x*l +
						poly->points[j].vect->x;
				result->points[k].vect->y = dir.y*l +
						poly->points[j].vect->y;
				result->points[k].vect->z = dir.z*l +
						poly->points[j].vect->z;
				
				/* now with the second one */
				subV( &poly->points[j].vect,
					&poly->points[NEXT(j)].vect, &dir );
				l = (-plane[i]->d - h)/(plane[i]->a*dir.x +
					plane[i]->b*dir.y + plane[i]->c*dir.z);
				result->points[k].vect->x = dir.x*l +
						poly->points[j].vect->x;
				result->points[k].vect->y = dir.y*l +
						poly->points[j].vect->y;
				result->points[k].vect->z = dir.z*l +
						poly->points[j].vect->z;

			}
			result->points[k].vect->x = dir.x*l +
					poly->points[j].vect->x;
			result->points[k].vect->y = dir.y*l +
					poly->points[j].vect->y;
			result->points[k].vect->z = dir.z*l +
					poly->points[j].vect->z;
		}
	}
	/* do it again for the first and last points */
	for ( i = 0; plane[i] != NULL; i++ ) {
		h = plane[i]->a*poly->points[j].vect->x +
		    plane[i]->b*poly->points[j].vect->y +
		    plane[i]->c*poly->points[j].vect->z;
		if ( h + plane[i]->d < 0 ) {
			/* point is on wrong side of the plane, so 
			 it'll be replaced by the intersection */
			
			/* get direction of straight line */
			subV( &poly->points[j].vect,
				&poly->points[j+1].vect, &dir );
			/* l comes from: X = (x0,y0,z0) + l(a,b,c) */
			l = (-plane[i]->d - h)/(plane[i]->a*dir.x +
				plane[i]->b*dir.y + plane[i]->c*dir.z);
			poly->points[j].vect->x = dir.x*l +
					poly->points[j].vect->x;
			poly->points[j].vect->y = dir.y*l +
					poly->points[j].vect->y;
			poly->points[j].vect->z = dir.z*l +
					poly->points[j].vect->z;
		}
	}

	return r;
}

int clipLineP ( PLANE_UL *plane, VECTOR_UL *start, VECTOR_UL *end ) {
	
}

#endif
