/* 
Unlimited Simulator is copyrighted (c) Bruno Barberi Gnecco, 1999, and
distributed under the GNU General Public License (GPL). The conditions can be
found in the file GPL, resident at the main directory of the distribution.
*/
#ifdef CLIENT
 
#include "ulengine.h"
#include <malloc.h>

ggi_visual_t	vis;
ggi_mode 	mode;
ggi_pixel	black, white;
OUTPUT_UL	im[2];
Bool		db;		/* using buffers? */
sByte		numframes;	/* how many, 1 or 2? */
ENGPREF_UL	engpref;

/* init engine. Pass 0, 0, 0, False, True if you want defaults. 
   Single is for single buffering */
sInt initULEngine ( uInt width, uInt height, uInt bits, Bool single, Bool usedb ) {
	sInt		i;
	ggi_color	col;

	/* set defaults */
	engpref.aa = False;
	engpref.fog.type = NONE;
	engpref.gtype = FLAT;

	if ( ggiInit() ) {
		fprintf( stderr, "Could not start GGI\n" );
		return FATALERROR;
	}

	if ( !(vis = ggiOpen( NULL )) ) {
		fprintf( stderr, "Could not open Visual\n" );
		return FATALERROR; 
	}
	
	ggiSetFlags( vis, GGIFLAG_ASYNC );

	/* now ggi initialization... */
	mode.frames = ( single == True ? 1 : 2 ); 
	mode.visible.x = ( width == 0 ? GGI_AUTO : width );
	mode.visible.y = ( height == 0 ? GGI_AUTO : height );
	mode.virt.x = mode.virt.y = GGI_AUTO;
	switch ( bits ) {
		case 8:
			mode.graphtype = GT_8BIT;
			break;
		case 15:
			mode.graphtype = GT_15BIT;
			break;
		case 16:
			mode.graphtype = GT_16BIT;
			break;
		case 24:
			mode.graphtype = GT_24BIT;
			break;
		case 32:
			mode.graphtype = GT_32BIT;
			break;
		default:
			fprintf( stderr, "Mode depth not supported: %d bits. Let GGI pick one.\n", bits );
		case 0:
			mode.graphtype = GGI_AUTO;
			break;
	}
	mode.dpp.x = mode.dpp.y = GGI_AUTO;

	/* CheckMode will fill a mode that should work, in case ours doesn't */
	ggiCheckMode( vis, &mode );

	if ( ggiSetMode( vis, &mode ) ) {
		fprintf( stderr, "Could not set video mode.\n" );
		return FATALERROR;
	}

	if (GT_SCHEME(mode.graphtype) == GT_PALETTE) {
		fprintf( stderr, "Palette mode not supported yet.\n" );
		return FATALERROR;
	}
		
	if ( mode.frames != 2 ) {
		fprintf( stderr, "Using single buffering...\n" );
	}

	/* test frames */
#ifdef NOAERA
	for ( i = 0; i < mode.frames; i++ )
		if ( ggiSetWriteFrame( vis, i ) != 0 || 
			ggiSetDisplayFrame( vis, i ) != 0 ) {
			fprintf( stderr, "Could not write to or display frame %d\n", i );
			ggiClose( vis );
			ggiExit();
			return FATALERROR;
		}
#endif

if ( usedb == True ) {
	/* get buffer */
	if ( !(im[0].buf = ggiDBGetBuffer( vis, 0 )) || 
	     ( single == True ? 0 != 0 : !(im[1].buf = ggiDBGetBuffer( vis, 1 ))) ) {
		fprintf( stderr, "Could not create buffers! "
				"Try to use single buffers or not to use them\n" );
		ggiClose( vis );
		ggiExit();
		return FATALERROR;
	}
	
	if ( !(im[0].buf->type & GGI_DB_SIMPLE_PLB) ||
	     ( single == True ? 0 != 0 : !(im[1].buf->type & GGI_DB_SIMPLE_PLB) ) ) {
		fprintf( stderr, "Non linear buffers not supported\n" );
		ggiClose( vis );
		ggiExit();
		return FATALERROR;
	}

	for ( i = 0; i < mode.frames; i++ ) {
		if ( ggiResourceAcquire( im[i].buf->resource, GGI_ACTYPE_WRITE ) ) {
			fprintf( stderr, "Couldn't write to buffer\n" );
			ggiClose( vis );
			ggiExit();
			return FATALERROR;
		}
			
		/* some shortcuts */
		im[i].stride = &im[i].buf->buffer.plb.stride;
		im[i].pix = im[i].buf->buffer.plb.pixelformat;
		im[i].fbptr = im[i].buf->write;
	}
}

	numframes = mode.frames;
	db = usedb;
	
	framenum = 0;
	/* initialization routines */
	
	col.r = col.g = col.b = 0;
	black = ggiMapColor( vis, &col );
	col.r = col.g = col.b = 0xFFFF;
	white = ggiMapColor( vis, &col );

/***** need to free memory here and in some cases above */
	initTrigTable();
	if ( initProject( mode.visible.x, mode.visible.y ) != OK ||
		initGraphic() != OK ) {
		endULEngine();
		return FATALERROR;
	}
	return OK;
}

/* free memory and close GGI */
void endULEngine ( void ) {
	sInt i;
	DOODAD_UL *dood;
	PARALLEL_UL *par;
	SPOTLIGHT_UL *spot;

	/* close routines */
	closeProject();
	closeGraphic();

	/* direct buffer, if exists */
	if ( db )
		for ( i = 0; i < numframes; i++ )
			ggiResourceRelease( im[i].buf->resource );
	
	/* free the linked lists */
	for ( dood = firstdoodad; firstdoodad != NULL; dood = firstdoodad )
		freeDoodad( dood );
	for ( par = firstparallel; firstparallel != NULL; par = firstparallel )
		freeParallel( par );
	for ( spot = firstspot; firstspot != NULL; spot = firstspot )
		freeSpot( spot );

	/* thanks for your help, ggi */
	ggiClose( vis );
	ggiExit();
	
	/* we're off! */
}

void setGraphics ( GRAPHIC_UL gtype ) {
	engpref.gtype = gtype;
}

void setAntialiasing ( Bool setAA ) {
	engpref.aa = setAA;
}

#endif
