/* 
Unlimited Simulator is copyrighted (c) Bruno Barberi Gnecco, 1999, and
distributed under the GNU General Public License (GPL). The conditions can be
found in the file GPL, resident at the main directory of the distribution.
*/
#ifdef CLIENT

#include "ulengine.h"
#include <malloc.h>

PARALLEL_UL *firstparallel;

/* PARALLEL lights are considered to be in the infinite and propagate light
rays in the VECTOR direction of RGB color. You can precompute vertices colors
in any mode, and polygons colors if you're using FLAT modes. (see update.c) */

PARALLEL_UL *createParallel ( VECTOR_UL *direction, ggi_color color ) {
	PARALLEL_UL *tmp, *light;
	
	if ( (light = (PARALLEL_UL *)malloc(sizeof(PARALLEL_UL))) == NULL )
		return NULL;
	
	copyV( &(light->dir), direction );
	copyColor( &light->color, &color );
	if ( !firstparallel ) {
		firstparallel = light;
		return light;
	}
	for ( tmp = firstparallel; tmp->next != NULL; tmp = tmp->next )
		tmp->next = light;
	light->next = NULL;
	return light; 
}

void setParallel ( PARALLEL_UL *light, VECTOR_UL *direction, ggi_color color ) {
	if ( direction != NULL )
		copyV( &(light->dir), direction );
	if ( light != NULL )
		copyColor( &light->color, &color );
}

void freeParallel ( PARALLEL_UL *light ) {
	PARALLEL_UL *tmp;
	
	if ( firstparallel == light ) {
		firstparallel = light->next;
	}
	else {
		for ( tmp = firstparallel; tmp->next != light; tmp = tmp->next )
			tmp->next = light->next;
	}
	free( light );
}

SPOTLIGHT_UL *firstspot;

/* angle is the opening (solid) angle */
SPOTLIGHT_UL *createSpot ( VECTOR_UL *pos, VECTOR_UL *direction, dFloat angle,
		ggi_color color, Bool volumetric ) {
	SPOTLIGHT_UL *tmp, *light;
	
	if ( !(light = (SPOTLIGHT_UL *)malloc(sizeof(SPOTLIGHT_UL))) )
		return NULL;
	
	copyV( &(light->dir), direction );
	copyV( &(light->pos), pos );
	copyColor( &light->color, &color );
	light->angle = angle;
	if ( !firstspot ) {
		firstspot = light;
		return light;
	}
	for ( tmp = firstspot; tmp->next != NULL; tmp = tmp->next )
		tmp->next = light;
	light->next = NULL;
	return light; 
}

void setSpot ( SPOTLIGHT_UL *light, VECTOR_UL *pos, VECTOR_UL *direction, 
		dFloat angle, ggi_color color, Bool volumetric ) {
	if ( direction != NULL )
		copyV( &(light->dir), direction );
	if ( pos != NULL )
		copyV( &(light->pos), pos );
	if ( light != NULL )
		copyColor( &light->color, &color );
	if ( angle != 0 )
		light->angle = angle;
	light->volumetric = volumetric;
}

void freeSpot ( SPOTLIGHT_UL *light ) {
	SPOTLIGHT_UL *tmp;
	
	if ( firstspot == light ) {
		firstspot = light->next;
	}
	else {
		for ( tmp = firstspot; tmp->next != light; tmp = tmp->next )
			tmp->next = light->next;
	}
	free( light );
}

SPOTLIGHT_UL *copySpot ( SPOTLIGHT_UL *light ) {
	return createSpot( &light->pos, &light->dir, light->angle, light->color,
			light->volumetric );
}
#endif
